//*****************************
//********* Includes **********
//*****************************

#ifdef AVR
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#else
#include "inttypes.h"
#endif
#include "softtimer.h"

//*****************************
//********* Constants *********
//*****************************
#define F_CPU 16000000UL
#ifndef F_CPU
#error "F_CPU not defined, see makefile..."
#endif


#if   ( F_CPU == 8000000 )
#elif ( F_CPU == 16000000 )
#else
#error "Keine Timereinstellungen fr F_CPU definiert !!!"
#endif

#ifdef AVR
// Einstellungen passen fr F_CPU = 8000000 Hz und F_CPU = 16000000 Hz
// Timertick = 1ms (0.001F sec.)
#define kTimer0Reload               (0.001F * (F_CPU / 64 ))   // CTC (Autoreload) mode   // 6    //( 256 - (0.001F * (F_CPU / 64 )) )
#define kTimer0Stop                 0
#define kTimer0StartPrescale        ( _BV (WGM01) | _BV (CS01) | _BV (CS00) ) // Prescaler 64, CTC (Autoreload) Mode

#else

#define kTimer0Reload               64  
#define kTimer0Stop                 0
#define kTimer0StartPrescale  
      
#endif



static uint8_t gSofttimerX;

static uint16_t gSofttimerFlags; // fr 16 Timer uint16_t und den enum-Type im Headerfile erweitern
static int16_t  gSofttimerValue[Softtimer_kMaxNumberOfTimers];



//***************************************************************************
//*
//* FUNCTION
//* Softtimer__Init - Initialisierung der Softwaretimer
//*
//***************************************************************************
void Softtimer__Init (void)
{
    uint8_t f;

    for (f=0; f<Softtimer_kMaxNumberOfTimers; f++)
    {
        gSofttimerValue[f] = 0;
    }

    gSofttimerFlags = 0x00;

    gSofttimerX = 0;

#ifdef AVR

    OCR0 = kTimer0Reload;

    TIMSK |= _BV (OCIE0);             // Timer 0 Output Compare INT freigeben

    TCCR0 = kTimer0StartPrescale;     // Start Timer 0 with prescaling f/64 in CTC mode

#endif

}
// END Softtimer__Init


//***************************************************************************
//*
//* INTERRUPT
//* Softtimer__IRQ - Timer0 Overflow Interrupt
//*
//***************************************************************************
#ifdef AVR

ISR (TIMER0_COMP_vect)
{

//    TCCR0 = kTimer0Stop;    // Timer 0 kurz anhalten

    gSofttimerX++;

//    TCCR0 = kTimer0StartPrescale;    // Timer 0 wieder starten

    //Display__IRQ ();

    SLM1606_update();

}
// END Softtimer__IRQ
#else

void Timer_Interrupt (void)
{
   gSofttimerX++;
}
#endif
//***************************************************************************
//*
//* FUNCTION
//* Softtimer__Handler - bearbeitet die SoftwareTimer
//*
//***************************************************************************

//Das Programm   T I M E R   verwaltet bis zu 8 Softwaretimer.
//Zeitbasis fuer die Softwaretimer ist Timer0 oder Timer1 des 8051.
//Er muss in Modus 2 (Reload) mit einem Reload-Wert von -250 arbeiten und
//einen Timer-Int. ausloesen (alle 250 us bei 12 MHz, bei anderem Quarz Reload-
//Wert korr.). Innerhalb der Int.-Routine muss das DATAByte TIMERX inc. werden.
//Hat TIMERX den Wert 4 erreicht (1 ms verstrichen), so werden alle aktiven
//Softwaretimer dec..
//Jeder Softwaretimer hat eine Groesse von 16 Bit und kann somit bis zu
//65,535 s zaehlen. Gestartet wird der Timer durch Setzen seines Bits in
//TIMEFG (Bit 0 fuer TIMER0, Bit 1 fuer TIMER1 usw.). Ist der Timer fertig,
//so loescht er sein Bit wieder.


void Softtimer__Handler (void)
{
static uint8_t  softtimerMem;
uint8_t  f;
uint16_t flag;

    softtimerMem = gSofttimerX;

    // 1 ms verstrichen ?
    if ( softtimerMem >= 2)
    {

        gSofttimerX -= softtimerMem;                      // Bit 0 und 1 von gSofttimerX unveraendert lassen

        flag = 0x0001;

        for (f=0; f<Softtimer_kMaxNumberOfTimers; f++)
        {
            if ( gSofttimerFlags & flag )
            {
                gSofttimerValue[f]--;
                if ( gSofttimerValue[f] <= 0 )
                    gSofttimerFlags &= ~flag;
            }

            flag <<= 1;
        }
    }
}
// END Softtimer__Handler





void Softtimer__Load ( Softtimer_ETimer timer, uint16_t milliSecs )
{
    gSofttimerValue[timer] = milliSecs;
    gSofttimerFlags |= (1 << timer);    // Timer starten
}


uint8_t Softtimer__Expired ( Softtimer_ETimer timer )
{
    if ( gSofttimerFlags &  (1 << timer) )
        return (0);
    else
        return (1);
}

